@@ -393,8 +393,11 @@ impl Address {
     }
 
     pub fn read(&self, recref: &RecRef, segment: SegmentId) -> PRes<Option<(u64, u16)>> {
-        let mut page = self.allocator.load_page(recref.page)?;
-        page.segment_read_entry(segment, recref.pos)
+        if let Some(mut page) = self.allocator.load_page_not_free(recref.page)? {
+            Ok(page.segment_read_entry(segment, recref.pos)?)
+        } else {
+            Ok(None)
+        }
     }
 
     fn get_or_insert_mut<'a>(&self, map: &'a mut HashMap<u64, Page>, k: u64) -> PRes<&'a mut Page> {
