@@ -33,7 +33,9 @@ impl Cache {
 
     fn put(&mut self, key: u64, value: ReadPage) {
         self.size += 1 << value.get_size_exp();
-        self.cache.insert(key, value);
+        if let Some(pre) = self.cache.insert(key, value) {
+            self.size -= 1 << pre.get_size_exp();
+        }
         while self.size > self.limit {
             if let Some(en) = self.cache.pop_front() {
                 self.size -= 1 << en.1.get_size_exp();
@@ -42,8 +44,11 @@ impl Cache {
             }
         }
     }
+
     fn remove(&mut self, key: u64) {
-        self.cache.remove(&key);
+        if let Some(removed) = self.cache.remove(&key) {
+            self.size -= 1 << removed.get_size_exp();
+        }
     }
 }
 
@@ -551,4 +556,30 @@ mod tests {
         let read_list = FreeList::read_free_list(&buf);
         assert_eq!(list, read_list);
     }
+
+    #[test]
+    fn test_cache_limit_remove() {
+        let mut cache = Cache::new(1050 as u64);
+        cache.put(10, ReadPage::new(Arc::new(Vec::new()), 0, 10, 9));
+        cache.put(20, ReadPage::new(Arc::new(Vec::new()), 0, 10, 9));
+        cache.put(30, ReadPage::new(Arc::new(Vec::new()), 0, 10, 9));
+        assert!(cache.size < 1050);
+        assert_eq!(cache.cache.len(), 2);
+        cache.remove(20);
+        cache.remove(30);
+        assert_eq!(cache.size, 0);
+    }
+
+    #[test]
+    fn test_cache_replace_limit_stay() {
+        let mut cache = Cache::new(100000 as u64);
+        cache.put(10, ReadPage::new(Arc::new(Vec::new()), 0, 10, 9));
+        cache.put(20, ReadPage::new(Arc::new(Vec::new()), 0, 10, 9));
+        cache.put(30, ReadPage::new(Arc::new(Vec::new()), 0, 10, 9));
+        let pre = cache.size;
+        cache.put(20, ReadPage::new(Arc::new(Vec::new()), 0, 10, 9));
+        assert_eq!(cache.size, pre);
+        cache.put(20, ReadPage::new(Arc::new(Vec::new()), 0, 10, 10));
+        assert_eq!(cache.size, pre - (1 << 9) + (1 << 10));
+    }
 }
