@@ -21,6 +21,8 @@ pub trait UpdateList {
 pub trait Device: Sync + Send {
     fn load_page(&self, page: u64) -> PRes<ReadPage>;
 
+    fn load_page_if_exists(&self, page: u64) -> PRes<Option<ReadPage>>;
+
     /// Load a page avoiding to skip base page metadata, used for root page or metadata
     /// manipulation.
     ///
@@ -286,6 +288,8 @@ impl Page {
     pub fn reset(&mut self) -> PRes<()> {
         self.buff = vec![0; self.buff.len()];
         self.buff[0] = self.exp;
+        let end = self.buff.len() - 1;
+        self.buff[end] = self.exp;
         Ok(())
     }
 
@@ -529,6 +533,15 @@ impl Device for DiscRef {
         Ok(ReadPage::new(Arc::new(ve), 2, page, exp[0]))
     }
 
+    fn load_page_if_exists(&self, page: u64) -> PRes<Option<ReadPage>> {
+        {
+            if *self.size.lock()? <= page {
+                return Ok(None);
+            }
+        }
+        Ok(Some(self.load_page(page)?))
+    }
+
     fn load_page_raw(&self, page: u64, size_exp: u8) -> PRes<Page> {
         use std::os::unix::prelude::FileExt;
         let size = (1 << size_exp) as u64; //EXP - (size_exp+size_mitigator);
@@ -653,6 +666,14 @@ impl Device for DiscRef {
         load_page(&mut self.file.lock()?.file, page)
     }
 
+    fn load_page_if_exists(&self, page: u64) -> PRes<Option<ReadPage>> {
+        let mut lock = self.file.lock()?;
+        if lock.file.len()? <= page {
+            return Ok(None);
+        }
+        Ok(Some(load_page(&mut lock.file, page)?))
+    }
+
     fn load_page_raw(&self, page: u64, size_exp: u8) -> PRes<Page> {
         load_page_raw(&mut self.file.lock()?.file, page, size_exp)
     }
@@ -720,6 +741,14 @@ impl Device for MemRef {
         load_page(&mut *self.data.lock()?, page)
     }
 
+    fn load_page_if_exists(&self, page: u64) -> PRes<Option<ReadPage>> {
+        let mut data = self.data.lock()?;
+        if data.len()? <= page {
+            return Ok(None);
+        }
+        Ok(Some(load_page(&mut *data, page)?))
+    }
+
     fn load_page_raw(&self, page: u64, size_exp: u8) -> PRes<Page> {
         load_page_raw(&mut *self.data.lock()?, page, size_exp)
     }
